{$ASMMODE INTEL}

unit fat;

interface
type
	TCurrentDir = packed record 
		Name: array [0..10] of char;
		Attribute: char;
		NTRes: char;
		A: char;
		CrtTime: word;
		CrtDate: word;
		LastAccessDate: word;
		FirstClusterHi: word;
		LastWriteTime: word;
		LastWriteDate: word;
		FirstClusterLo: word;
		FileSize: dword;
	end;

var
	BootSector: packed record
		SysName: array [0..7] of char;
		BytesPerSector: word;
		SectorsPerCluser: byte;
		ReservedSectors: word;
		NumberFATs: byte;
		MaxRootEntries: word;
		TotalSectorNumber: word;
		Media: byte;
		FATSectorNumber: word;
		SectorsPerTrack: word;
		HeadNumber: word;
		HiddenSectors: dword;
		TotalSectorNumber32: dword;
		DriveNumber: byte;
		Reserved1: char;
		BootSignature: char;
		VolumeSerial: dword;
		VolumeLabel: array [0..10] of char;
		FileSystemType: array [0..7] of char;
	end;

	FATCell: array [0..4095] of word;
	CurrentDir: array [0..223] of TCurrentDir;
	PathCurrentDir: pchar ='/';
	RootDir: array [0..65535] of char;

procedure LoadBootSector;
procedure LoadDir(path: pchar);
function ClusterToSector(cluster: integer): integer;
procedure ReadFile(Name: pchar);
function FindFile(Name: pchar): integer;
procedure LoadFAT;
procedure List(dir:pchar);

implementation

uses floppy, console;

procedure LoadFAT;
var
i:integer=0;
ii:integer=0;
begin
	LoadBootSector;
	for i:=0 to 511 do
		Buffer[i]:=#0;

	FloppyRead(0, BootSector.ReservedSectors, BootSector.FATSectorNumber);
	for i:=0 to 1536 do
	begin
		FATCell[i*2]:= ord(byte(Buffer[ii+1]){>>4} and $0F) << 8 or ord(Buffer[ii]);
		FATCell[i*2+1]:= ord(Buffer[ii+2]) << 4 or ord(byte(Buffer[ii+1])>>4 and $0F);
		ii:=ii+3;
	end;
end;

procedure LoadBootSector;
var
i: integer = 0;
begin
	for i:=0 to 511 do
		Buffer[i]:=#0;
	FloppyRead(0, 0, 1);
	BootSector.SysName[0] := Buffer[3];
	BootSector.SysName[1] := Buffer[4];
	BootSector.SysName[2] := Buffer[5];
	BootSector.SysName[3] := Buffer[6];
	BootSector.SysName[4] := Buffer[7];
	BootSector.SysName[5] := Buffer[8];
	BootSector.SysName[6] := Buffer[9];
	BootSector.SysName[7] := Buffer[10];
	BootSector.BytesPerSector := ord(Buffer[12]) << 8 or ord(Buffer[11]);
	BootSector.SectorsPerCluser := ord(Buffer[13]);
	BootSector.ReservedSectors := ord(Buffer[15]) << 8 or ord(Buffer[14]);
	BootSector.NumberFATs := ord(Buffer[16]);
	BootSector.MaxRootEntries := ord(Buffer[18]) << 8 or ord(Buffer[17]);
	BootSector.TotalSectorNumber := ord(Buffer[20]) << 8 or ord(Buffer[19]);
	BootSector.Media := ord(Buffer[21]);
	BootSector.FATSectorNumber := ord(Buffer[23]) << 8 or ord(Buffer[22]);
	BootSector.SectorsPerTrack := ord(Buffer[25]) << 8 or ord(Buffer[24]);
	BootSector.HeadNumber := ord(Buffer[27]) << 8 or ord(Buffer[26]);
	BootSector.HiddenSectors := ord(Buffer[31]) << 24 or ord(Buffer[30]) << 16 or ord(Buffer[29]) << 8 or ord(Buffer[28]);
	BootSector.TotalSectorNumber32 := ord(Buffer[35]) << 24 or ord(Buffer[34]) << 16 or ord(Buffer[33]) << 8 or ord(Buffer[32]);
	BootSector.DriveNumber := ord(Buffer[36]);
	BootSector.Reserved1 := Buffer[37];
	BootSector.BootSignature := Buffer[38];
	BootSector.VolumeSerial := ord(Buffer[42]) << 24 or ord(Buffer[41]) << 16 or ord(Buffer[40]) << 8 or ord(Buffer[39]);
	BootSector.VolumeLabel[0] := Buffer[43];
	BootSector.VolumeLabel[1] := Buffer[44];
	BootSector.VolumeLabel[2] := Buffer[45];
	BootSector.VolumeLabel[3] := Buffer[46];
	BootSector.VolumeLabel[4] := Buffer[47];
	BootSector.VolumeLabel[5] := Buffer[48];
	BootSector.VolumeLabel[6] := Buffer[49];
	BootSector.VolumeLabel[7] := Buffer[50];
	BootSector.VolumeLabel[8] := Buffer[51];
	BootSector.VolumeLabel[9] := Buffer[52];
	BootSector.VolumeLabel[10] := Buffer[53];
	BootSector.FileSystemType[0] := Buffer[54];
	BootSector.FileSystemType[1] := Buffer[55];
	BootSector.FileSystemType[2] := Buffer[56];
	BootSector.FileSystemType[3] := Buffer[57];
	BootSector.FileSystemType[4] := Buffer[58];
	BootSector.FileSystemType[5] := Buffer[59];
	BootSector.FileSystemType[6] := Buffer[60];
	BootSector.FileSystemType[7] := Buffer[61];

end;

procedure LoadDir(path: pchar);
var 
	i: integer =0;
	a: integer =0;
begin
	if (path[0] = '/') and (path[1] = #0) then
	begin
		for a:=0 to 511 do
			Buffer[a]:=#0;
		for i:=0 to 13 do
		begin
			FloppyRead(0, (BootSector.ReservedSectors + (BootSector.FATSectorNumber * BootSector.NumberFATs)) + i, 1);
			for a:=0 to 15 do
			begin
				CurrentDir[a+i*16].Name[0] := Buffer[a*32+0];
				CurrentDir[a+i*16].Name[1] := Buffer[a*32+1];
				CurrentDir[a+i*16].Name[2] := Buffer[a*32+2];
				CurrentDir[a+i*16].Name[3] := Buffer[a*32+3];
				CurrentDir[a+i*16].Name[4] := Buffer[a*32+4];
				CurrentDir[a+i*16].Name[5] := Buffer[a*32+5];
				CurrentDir[a+i*16].Name[6] := Buffer[a*32+6];
				CurrentDir[a+i*16].Name[7] := Buffer[a*32+7];
				CurrentDir[a+i*16].Name[8] := Buffer[a*32+8];
				CurrentDir[a+i*16].Name[9] := Buffer[a*32+9];
				CurrentDir[a+i*16].Name[10] := Buffer[a*32+10];

				CurrentDir[a+i*16].Attribute := Buffer[a*32+11];
				CurrentDir[a+i*16].NTRes := Buffer[a*32+12];
				CurrentDir[a+i*16].A := Buffer[a*32+13];
				CurrentDir[a+i*16].CrtTime := ord(Buffer[a*32+15]) << 8 or ord(Buffer[a*32+14]);
				CurrentDir[a+i*16].CrtDate := ord(Buffer[a*32+17]) << 8 or ord(Buffer[a*32+16]);
				CurrentDir[a+i*16].LastAccessDate := ord(Buffer[a*32+19]) << 8 or ord(Buffer[a*32+18]);
				CurrentDir[a+i*16].FirstClusterHi := ord(Buffer[a*32+21]) << 8 or ord(Buffer[a*32+20]);
				CurrentDir[a+i*16].LastWriteTime := ord(Buffer[a*32+23]) << 8 or ord(Buffer[a*32+22]);
				CurrentDir[a+i*16].LastWriteDate := ord(Buffer[a*32+25]) << 8 or ord(Buffer[a*32+24]);
				CurrentDir[a+i*16].FirstClusterLo := ord(Buffer[a*32+27]) << 8 or ord(Buffer[a*32+26]);
				CurrentDir[a+i*16].FileSize := ord(Buffer[a*32+29]) << 24 or ord(Buffer[a*32+28]) << 16 or ord(Buffer[a*32+27]) << 8 or ord(Buffer[a*32+29]);
			end;
		end;
	end;
end;

function ClusterToSector(cluster: integer): integer;
begin
	ClusterToSector:= (BootSector.ReservedSectors + (BootSector.FATSectorNumber * BootSector.NumberFATs)) + ((BootSector.MaxRootEntries * 32) div 512) - 2 + cluster;
end;

function FindFile(Name: pchar): integer;
var
	i: integer;
begin
	for i:=0 to 223 do
	begin
		if (CurrentDir[i].Name[0] = Name[0]) and 
		(CurrentDir[i].Name[1] = Name[1]) and
		(CurrentDir[i].Name[2] = Name[2]) and
		(CurrentDir[i].Name[3] = Name[3]) and
		(CurrentDir[i].Name[4] = Name[4]) and
		(CurrentDir[i].Name[5] = Name[5]) and
		(CurrentDir[i].Name[6] = Name[6]) and
		(CurrentDir[i].Name[7] = Name[7]) and
		(CurrentDir[i].Name[8] = Name[8]) and
		(CurrentDir[i].Name[9] = Name[9]) and
		(CurrentDir[i].Name[10] = Name[10]) then
		begin
			FindFile := i;
			Exit;
		end;
	end;
	PrintStr('Nie odnaleziono pliku');
end;

procedure ReadFile(Name: pchar);
var
a :integer;
CurrentCluster:integer;
begin
	LoadBootSector;
	LoadFAT;
	LoadDir('/');
	for a:=0 to 511 do
		Buffer[a]:=#0;
	FloppyRead(0, ClusterToSector(CurrentDir[FindFile(Name)].FirstClusterLo) , 1);
	CurrentCluster:=CurrentDir[FindFile(Name)].FirstClusterLo;
	PrintStr(#13);
	for a:=0 to 511 do
		PrintChar(Buffer[a]);

	while (FATCell[CurrentCluster] <> $FFF) do
	begin
		for a:=0 to 511 do
			Buffer[a]:=#0;
		FloppyRead(0, ClusterToSector(FATCell[CurrentCluster]) , 1);
		CurrentCluster:=FATCell[CurrentCluster];
		for a:=0 to 511 do
			PrintChar(Buffer[a]);
	end;
end;

procedure List(dir:pchar);
var
	b:integer=0;
begin
	LoadBootSector;
	LoadDir(dir);
	repeat
		if (CurrentDir[b].Name[0] <> char($E5)) and (CurrentDir[b].Attribute <> char($0F)) then
		begin
			PrintStr(#13);
			PrintChar(CurrentDir[b].Name[0]);
			PrintChar(CurrentDir[b].Name[1]);
			PrintChar(CurrentDir[b].Name[2]);
			PrintChar(CurrentDir[b].Name[3]);
			PrintChar(CurrentDir[b].Name[4]);
			PrintChar(CurrentDir[b].Name[5]);
			PrintChar(CurrentDir[b].Name[6]);
			PrintChar(CurrentDir[b].Name[7]);
			PrintChar(CurrentDir[b].Name[8]);
			PrintChar(CurrentDir[b].Name[9]);
			PrintChar(CurrentDir[b].Name[10]);
		end;
		b:=b+1;
	until CurrentDir[b].Name[0] = char($00);
end;

end.