{$ASMMODE INTEL}

unit idt;

interface

const
	PIC1 = $20;
	PIC2 = $A0;
	ICW1 = $11;
	ICW4 = $01;
	TRAP_GATE = $8F00;

type
	TIDTEntry = packed record
		offset_0: word;
		selector: word;
		typ: word;
		offset_16: word;
	end;

	TIDTPtr = packed record
		limit : word;
		base : LongWord;
	end;

var
	IDTTables : array[0..255] of TIDTEntry;
	IDTPtr : TIDTPtr;

procedure setup_int(i: integer; p_hand:dword; typ:word);
procedure init_idt;
procedure init_pic;
procedure keys_isr; external 'int.o';
procedure unknown_interrupt_isr; external 'int.o';
procedure timer_isr; external 'int.o';
procedure floppy_isr; external 'int.o';

implementation

procedure setup_int(i: integer; p_hand:dword; typ:word); [public, alias : 'setup_int'];
begin
	IDTTables[i].offset_0 := p_hand;
	IDTTables[i].selector := $08;
	IDTTables[i].typ := typ;
	IDTTables[i].offset_16 := (p_hand >> 16);
end;

procedure init_idt; [public, alias : 'init_idt'];
var
	i:integer;
begin
	for i:=0 to 255 do
	begin
		setup_int(i, DWord(@unknown_interrupt_isr), TRAP_GATE);
	end;
	setup_int(32, DWord(@timer_isr), TRAP_GATE);
	setup_int(33, DWord(@keys_isr), TRAP_GATE);
	setup_int(38, DWord(@floppy_isr), TRAP_GATE);
	IDTPtr.limit := (SizeOf(TIDTEntry)*256)-1;
	IDTPtr.base := DWord(@IDTTables);
	asm
		lidt [IDTPtr]
	end;
end;

procedure init_pic; [public, alias : 'init_pic'];
begin
	outportb(PIC1, ICW1);
	outportb(PIC2, ICW1);
	outportb(PIC1 + 1, pic1);
	outportb(PIC2 + 1, pic2);
	outportb(PIC1 + 1, 4);
	outportb(PIC2 + 1, 2);
	outportb(PIC1 + 1, ICW4);
	outportb(PIC2 + 1, ICW4);
	outportb(PIC1 + 1, 0);
	outportb(PIC2 + 1, 0);
end;

end.
